#############################################################################
##
#W  ideals-def.gd           Manuel Delgado <mdelgado@fc.up.pt>
#W                          Pedro Garcia-Sanchez <pedro@ugr.es>
#W                          Jose Morais <jjoao@netcabo.pt>
##
##
#H  @(#)$Id: ideals-def.gd,v 0.92 $
##
#Y  Copyright 2005 by Manuel Delgado,
#Y  Pedro Garcia-Sanchez and Jose Joao Morais
#Y  We adopt the copyright regulations of GAP as detailed in the 
#Y  copyright notice in the GAP manual.
##
#############################################################################

#############################################################################
##
#R  IsIdealOfNumericalSemigroupRep
##
##  The representation of an ideal of a numerical semigroup.
##
#############################################################################
DeclareRepresentation( "IsIdealOfNumericalSemigroupRep", 
        IsComponentObjectRep and IsAttributeStoringRep and IsAdditiveElement and IsAdditiveElementWithInverse and IsMultiplicativeElement,
        ["list"]);


#############################################################################
##
#C  IsIdealOfNumericalSemigroup
##
##  The category of ideals of numerical semigroups.
##
#############################################################################
DeclareCategory( "IsIdealOfNumericalSemigroup", IsObject and IsIdealOfNumericalSemigroupRep);


#############################################################################
##
#C  IsIdealOfNumericalSemigroup
##
##  The family category of ideals of numerical semigroups.
##
#############################################################################
DeclareCategoryFamily( "IsIdealOfNumericalSemigroup" );


#############################################################################
##
#C  IsIdealOfNumericalSemigroup
##
##  The collections category of ideals of numerical semigroups.
##
#############################################################################
DeclareCategoryCollections( "IsIdealOfNumericalSemigroup" );



BindGlobal( "IdealsOfNumericalSemigroupsFamily",
        NewFamily( "IdealsOfNumericalSemigroups", IsIdealOfNumericalSemigroup ));

BindGlobal( "IdealsOfNumericalSemigroupsType",
        NewType( IdealsOfNumericalSemigroupsFamily, IsIdealOfNumericalSemigroup));



#############################################################################
##
#F IdealOfNumericalSemigroup(l,S)
##
## l is a list of integers and S a numerical semigroup
##
## returns the ideal of S generated by l.
##
#############################################################################
DeclareGlobalFunction("IdealOfNumericalSemigroup");


#############################################################################
##
#F  GeneratorsOfIdealOfNumericalSemigroup(I)
## 
##  Returns a set of generators of the ideal I. 
##  If a minimal generating system has already been computed, this
##  is the set returned.
############################################################################
DeclareGlobalFunction("GeneratorsOfIdealOfNumericalSemigroup");


#############################################################################
##
#F  GeneratorsOfIdealOfNumericalSemigroupNC(I)
## 
##  Returns a set of generators of the ideal I. 
############################################################################
DeclareGlobalFunction("GeneratorsOfIdealOfNumericalSemigroupNC");



#############################################################################
##
#F AmbientNumericalSemigroupOfIdeal(I)
##
##  Returns the ambient semigroup of the ideal I. 
############################################################################
DeclareGlobalFunction("AmbientNumericalSemigroupOfIdeal");



#############################################################################
##
#F SumIdealsOfNumericalSemigroup(I,J)
##
## returns the sum of the ideals I and J (in the same ambient semigroup)
#############################################################################
DeclareGlobalFunction("SumIdealsOfNumericalSemigroup");



#############################################################################
##
#F SubtractIdealsOfNumericalSemigroup(I,J)
##
## returns the ideal I - J
#############################################################################
DeclareGlobalFunction("SubtractIdealsOfNumericalSemigroup");


#############################################################################
##
#F  BelongsToIdealOfNumericalSemigroup(n,I)
## 
##  Tests if the integer n belongs to the ideal I.
##
#############################################################################
DeclareGlobalFunction("BelongsToIdealOfNumericalSemigroup");



#############################################################################
##
#F DifferenceOfdealsOfNumericalSemigroup(I,J)
##
## returns the set difference I\J (J must be contained in I)
#############################################################################
DeclareGlobalFunction("DifferenceOfIdealsOfNumericalSemigroup");



#############################################################################
##
#F MultipleOfIdealOfNumericalSemigroup(n,I)
##
## n is a non negative integer and I is an ideal
## returns the multiple nI (I+...+I n times) of I 
#############################################################################
DeclareGlobalFunction("MultipleOfIdealOfNumericalSemigroup");



#############################################################################
##
#F HilbertFunctionOfIdealOfNumericalSemigroup(n,I)
##
## returns the value of the Hilbert function associated to I in n,
## that is, nI\(n+1)I. I must be an ideal included in its ambient semigroup.
#############################################################################
DeclareGlobalFunction("HilbertFunctionOfIdealOfNumericalSemigroup");



#############################################################################
##
#F BlowUpIdealOfNumericalSemigroup(I)
##
##
#############################################################################
DeclareGlobalFunction("BlowUpIdealOfNumericalSemigroup");



#############################################################################
##
#A MinimalGeneratingSystemOfIdealOfNumericalSemigroup(I)
##
## The argument I is an ideal of a numerical semigroup
## returns the minimal generating system of I. 
## 
#############################################################################
DeclareAttribute("MinimalGeneratingSystemOfIdealOfNumericalSemigroup", IsIdealOfNumericalSemigroup);



#############################################################################
##
#A SmallElementsOfIdealOfNumericalSemigroup
##
##  Returns the list of elements in the ideal I up to the last gap + 1.
##
#############################################################################
DeclareAttribute( "SmallElementsOfIdealOfNumericalSemigroup", IsIdealOfNumericalSemigroup);



#############################################################################
##
#F  MaximalIdealOfNumericalSemigroup(S)
##
##  Returns the maximal ideal of S.
##
#############################################################################
DeclareGlobalFunction("MaximalIdealOfNumericalSemigroup");
